package task

import (
	"acme-mana/src/acme"
	"acme-mana/src/common"
	"acme-mana/src/conf"
	"encoding/json"
	"log"
	"os"
	"path"
	"time"
)

var AutoRefreshCertTicker = time.NewTicker(time.Hour)

func AutoRefreshCert() {
	log.Println("Start auto refresh cert")
	defer AutoRefreshCertTicker.Stop()
	for {
		select {
		case <-AutoRefreshCertTicker.C:
			doRefreshCert()
		}
	}
}

func doRefreshCert() {
	config := conf.Config()
	certs := config.Certs

	for _, cert := range *certs {
		doRefreshCertOne(&cert)
	}

	//domains := GetAppConfig().Domains
	//for _, domain := range domains {
	//	doRefreshCertOnce(domain)
	//}
}

func doRefreshCertOne(cert *conf.CertConf) {
	name := cert.Name
	dir := cert.Dir
	certDir := path.Join(dir, name)
	if !ValidExist(certDir, cert) {
		acme.Apply(cert)
		return
	}
	infoFile := path.Join(certDir, common.CertInfoFileName)
	certInfo := ParseCertInfo(infoFile, cert)
	log.Println("校验当前证书有效时间, 证书: {}", name)
	if certInfo.Info.NotAfter.Sub(time.Now()) < 14*24*time.Hour {
		log.Println("申请即将过期的证书: {}", name)
		acme.Apply(cert)
	}

}

//func doRefreshCertOnce(domain Domain) {
//	name := domain.Name
//	dir := GetAppConfig().CertDir
//	certDir := path.Join(dir, name)
//	if !ValidExist(certDir, domain) {
//		Apply(domain)
//	}
//	infoFile := path.Join(certDir, CertInfoFileName)
//	certInfo := ParseCertInfo(infoFile, domain)
//	log.Println("Checking if the certificate is expired, Domain: {}", name)
//	if certInfo.Info.NotAfter.Sub(time.Now()) < 14*24*time.Hour {
//		log.Println("Apply for a certificate that is about to expire, domain name:", name)
//		Apply(domain)
//	}
//}

func ValidExist(certDir string, cert *conf.CertConf) bool {
	_, err := os.Stat(certDir)
	if os.IsNotExist(err) {
		log.Printf("Applying for a certificate, Domain: %s certificate directory does not exist!", cert.Name)
		return false
	}
	if !ExistFile(certDir, common.CertFileName) {
		log.Printf("Applying for a certificate, Domain: %s %s does not exist!", cert.Name, common.CertFileName)
		return false
	}
	if !ExistFile(certDir, common.KeyFileName) {
		log.Printf("Applying for a certificate, Domain: %s %s does not exist!", cert.Name, common.KeyFileName)
		return false
	}
	if !ExistFile(certDir, common.CertInfoFileName) {
		log.Printf("Applying for a certificate, Domain: %s %s does not exist!", cert.Name, common.CertInfoFileName)
		return false
	}
	return true
}

func ParseCertInfo(infoFile string, cert *conf.CertConf) *acme.CertInfo {
	infoBytes, err := os.ReadFile(infoFile)
	if err != nil {
		log.Println("Failed to read cert info file, Domain: {}", cert.Name)
	}
	var certInfo acme.CertInfo
	err = json.Unmarshal(infoBytes, &certInfo)
	//if err != nil {
	//	log.Println("Failed to parse cert info file, Domain: {}", domain.Name)
	//}
	return &certInfo
}

func ExistFile(dir string, fileName string) bool {
	f := path.Join(dir, fileName)
	_, err := os.Stat(f)
	return !os.IsNotExist(err)

}
